/**************************************************************************
 * 
 *  MCICMDS.C           
 *
 *  MCI Optibase MPEG Driver         
 *
 *  Description:
 *
 *      MCI Command Message Procedures
 *
 ***************************************************************************/
            
#define  UNICODE
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include "mmddk.h"
#include <stdlib.h>
#include <string.h> 
#include <stdio.h>
#include "digitalv.h"
#include "mcioptb.h"
#include "common.h"           

GETSYSTEM lpfnTempGSB; //bartw: added externs for video client  
unsigned char	loop_terminate;

#define NO_LENGTH   0xFFFFFFFF      //Invalid length 

extern HINSTANCE       hModuleInstance;    // module instance  (different in NT - DLL instances)

// Forward references to non-exported functions
static DWORD NEAR PASCAL optbMciSet(int iInst, DWORD dwFlags, LPMCI_DGV_SET_PARMS lpSet);
static BOOL parse_file(int iInst, LPSTR lpstrElementName);
static BOOL parse_line(HFILE optbfile, LPSTR pointer, int buffer_size, char ending);  
static DWORD str_to_dw(LPSTR string);


/****************************************************************************
 * Function: int optbInstanceCreate - Create an OpenInstance
 *                       structure for a given MCI device ID.
 *
 * Parameters:
 *
 *      UINT uDeviceID - MCI device ID.
 *
 *      UINT iPort - Port index (0..3).
 *
 *      UINT iDev - Device index (0..6).
 *
 * Returns: a pointer to the OpenInstance structure created if
 *        successful, otherwise NULL.
 *
 *       Each time MCI uses this driver to open a device,
 *       optbInstanceCreate() is called to create an OpenInstance structure
 *       and associate it with the MCI device ID.
 ***************************************************************************/
int FAR PASCAL
optbInstanceCreate(UINT uDeviceID)
{
    int            iInst; 
    BYTE			achData[5];

    //
    // Create new "open instance" entry for the specified device
    //
    iInst = MemAllocInstance();

    if(iInst != -1)
    {
        pinst[iInst].uDeviceID        = uDeviceID;
        pinst[iInst].dwTimeFormat     = MCI_FORMAT_FRAMES;
        pinst[iInst].dwCounterFormat  = MCI_FORMAT_FRAMES;
        pinst[iInst].mcimode = MCI_MODE_STOP; //bartw: MCI_MODE_STOP? 
        optbDataPosition(pinst[iInst].position,VISCADATAHMSF,0,0,0,0);
        optbDataPosition(achData,VISCADATAHMSF,0,1,0,0);
        optbDataToMciTimeFormat(iInst, TRUE, achData, 
                    &(pinst[iInst].length));
      }

    // 0 is an illegal value for device id.
    if(uDeviceID != 0) 
        mciSetDriverData(uDeviceID, (UINT)iInst);

    return (iInst);
}



/****************************************************************************
 * Function: void optbInstanceDestroy - Destroy an OpenInstance.
 *
 * Parameters:
 *
 *      int iInst - Pointer to OpenInstance struct to
 *                       destroy.
 *
 *       When an MCI device ID is closed, optbInstanceDestroy() is called
 *       to free the OpenInstance structure corresponding to that device ID.
 *
 ***************************************************************************/
void FAR PASCAL
optbInstanceDestroy(int iInst)
{
    mciSetDriverData(pinst[iInst].uDeviceID, 0L); // prevent reenter if we yield in this function.

    _DPF4(DBG_MEM, "optbInstanceDestroy - Freeing iInst = %d \n", iInst);
    MemFreeInstance(iInst);
}
   
/****************************************************************************
 * Function: UINT optbDataPosition - Create a ViSCA position
 *              data structure.
 *
 * Parameters:
 *
 *      LPSTR lpstrData - Buffer to hold ViSCA data structure.
 *
 *      BYTE bTimeFormat - Time format.  May be one of:
 *              VISCADATAHMS, VISCADATAHMSF, VISCATIMECODENDF, and
 *              VISCADATATIMECODEDF.
 *
 *      BYTE bHours - Hours.
 *
 *      BYTE bMinutes - Minutes.
 *
 *      BYTE bSeconds - Seconds.
 *
 *      BYTE bFrames - Frames.
 *
 * Returns: length of data structure (5).
 ***************************************************************************/
UINT FAR PASCAL    
optbDataPosition(
                LPSTR   lpstrData,
                BYTE    bTimeFormat,
                BYTE    bHours,
                BYTE    bMinutes,
                BYTE    bSeconds,
                BYTE    bFrames)
{
    lpstrData[0] = bTimeFormat;
    lpstrData[1] = TOBCD(bHours);
    lpstrData[2] = TOBCD(bMinutes);
    lpstrData[3] = TOBCD(bSeconds);
    if(bTimeFormat == VISCADATAHMS)  
        lpstrData[4] = 0; /* We only support second accuracy */
    else
        lpstrData[4] = TOBCD(bFrames);
    return (5);
}
/****************************************************************************
 * Function: UINT optbMciFPS - Returns the number of frames per second
 *               for an MCI time format.
 *
 * Parameters:
 *
 *      DWORD dwTimeFormat - MCI time format.
 *
 * Returns: number of frames per second if successful, otherwise 0.
 *
 *       This function should only be used for SMPTE time formats,
 *       i.e. MCI_FORMAT_SMPTE_XX, where XX is 24, 25, 30, or 30DROP.
 ***************************************************************************/
static UINT NEAR PASCAL
optbMciFPS(DWORD dwMCITimeFormat)
{        
    switch (dwMCITimeFormat)
    {
        case MCI_FORMAT_SMPTE_24:
            return (24);
        case MCI_FORMAT_SMPTE_25:
            return (25);
        case MCI_FORMAT_SMPTE_30:
        case MCI_FORMAT_SMPTE_30DROP:
            return (30);
        default:
            return (0);
    }
}


/****************************************************************************
 * Function: DWORD optbMciTimeFormatToViscaData - Convert an Mci time
 *               value to a ViSCA data structure.
 *
 * Parameters:
 *
 *      int iInst - Instance on whose behalf conversion
 *                       is being done.
 *
 *      BOOL fTimecode - Are we using the timecode or counter? (can both be non-drop frame).
 *
 *      DWORD dwTime - Time value to convert.
 *
 *      LPSTR lpstrData - Buffer to hold result.
 *
 *      BYTE bDataFormat - ViSCA data format desired.
 *
 * Returns: an Mci error code.
 *
 *       Converts an MCI DWORD position variable in the current MCI time
 *       format (specified in iInst->dwTimeFormat) to a ViSCA 5-byte
 *       position data structure of the type specified by bDataFormat.
 ***************************************************************************/
DWORD FAR PASCAL
optbMciTimeFormatToViscaData(int iInst, BOOL fTimecode, DWORD dwTime, LPSTR lpstrData, BYTE bDataFormat)
{
    BYTE    bHours;
    BYTE    bMinutes;
    BYTE    bSeconds;
    BYTE    bFrames;
    UINT	uDevFPS = 30;   //bartw: assume that decoder plays 30 fps!
    DWORD   dwTimeFormat;

    if(fTimecode)
        dwTimeFormat =  pinst[iInst].dwTimeFormat;
    else
        dwTimeFormat =  pinst[iInst].dwCounterFormat;
    //
    // First extract hours, minutes, seconds, and frames from MCI data
    //
    switch (dwTimeFormat)
    {
        case MCI_FORMAT_MILLISECONDS:
            bHours   = (BYTE)(dwTime / 3600000L);
            bMinutes = (BYTE)((dwTime / 60000L) % 60);
            bSeconds = (BYTE)((dwTime /  1000L) % 60);
            bFrames  = (BYTE)((dwTime % 1000) * uDevFPS / 1000); 
            break;
        case MCI_FORMAT_HMS:
            bHours   = MCI_HMS_HOUR(dwTime);
            bMinutes = MCI_HMS_MINUTE(dwTime);
            bSeconds = MCI_HMS_SECOND(dwTime);
            bFrames  = 0;
            break;
        case MCI_FORMAT_MSF:
            bHours   = (BYTE)(MCI_MSF_MINUTE(dwTime) / 60);
            bMinutes = (BYTE)(MCI_MSF_MINUTE(dwTime) % 60);
            bSeconds = MCI_MSF_SECOND(dwTime);
            bFrames  = MCI_MSF_FRAME(dwTime);
            break;
        case MCI_FORMAT_TMSF:
            if (MCI_TMSF_TRACK(dwTime) != 1)
                return(MCIERR_OUTOFRANGE);
            bHours   = (BYTE)(MCI_TMSF_MINUTE(dwTime) / 60);
            bMinutes = (BYTE)(MCI_TMSF_MINUTE(dwTime) % 60);
            bSeconds = MCI_TMSF_SECOND(dwTime);
            bFrames  = MCI_TMSF_FRAME(dwTime);
            break;
        case MCI_FORMAT_FRAMES:
        case MCI_FORMAT_SAMPLES:
            bHours   = (BYTE)(dwTime / (uDevFPS * 3600L));
            bMinutes = (BYTE)((dwTime / (uDevFPS * 60)) % 60);
            bSeconds = (BYTE)((dwTime / uDevFPS) % 60);
            bFrames  = (BYTE)(dwTime % uDevFPS);
            break;
        case MCI_FORMAT_SMPTE_24:
        case MCI_FORMAT_SMPTE_25:
        case MCI_FORMAT_SMPTE_30:
        case MCI_FORMAT_SMPTE_30DROP:
            bHours   = LOBYTE(LOWORD(dwTime));
            bMinutes = HIBYTE(LOWORD(dwTime));
            bSeconds = LOBYTE(HIWORD(dwTime));
            bFrames  = (BYTE)(UINT)MulDiv(HIBYTE(HIWORD(dwTime)),
                                          uDevFPS,
                                          optbMciFPS(dwTimeFormat));
            //
            // Because of rounding, it's theoretically possible that bFrames
            // will exceed uDevFPS - 1.  So check for this condition.
            //
            if (bFrames >= uDevFPS)
                bFrames = uDevFPS - 1;
            break;          
        case MCI_FORMAT_BYTES:
        default:
            return (MCIERR_BAD_TIME_FORMAT);
    }
    //
    // Create ViSCA data
    //

    if( (bMinutes >= 60) || (bSeconds >= 60) || (bFrames >= uDevFPS) )
        return(MCIERR_OUTOFRANGE);

    // Smpte timecode has a maximum of 23:59:59:29

    if(fTimecode && (bHours >= 24))
        return(MCIERR_OUTOFRANGE);

    optbDataPosition(lpstrData, bDataFormat, bHours, bMinutes, bSeconds, bFrames);

    return (MCIERR_NO_ERROR);
}
 
DWORD FAR PASCAL
optbMilisecondsToViscaData(int iInst, BOOL fTimecode, DWORD dwTime, LPSTR lpstrData, BYTE bDataFormat)
{
    BYTE    bHours;
    BYTE    bMinutes;
    BYTE    bSeconds;
    BYTE    bFrames;
    UINT	uDevFPS = 30;   //bartw: assume that decoder plays 30 fps!
    DWORD   dwTimeFormat;


    dwTimeFormat =  pinst[iInst].dwTimeFormat;
 
    bHours   = (BYTE)(dwTime / 3600000L);
    bMinutes = (BYTE)((dwTime / 60000L) % 60);
    bSeconds = (BYTE)((dwTime /  1000L) % 60);
    bFrames  = (BYTE)((dwTime % 1000) * uDevFPS / 1000); 
  

    if( (bMinutes >= 60) || (bSeconds >= 60) || (bFrames >= uDevFPS) )
        return(MCIERR_OUTOFRANGE);

    // Smpte timecode has a maximum of 23:59:59:29

    if(fTimecode && (bHours >= 24))
        return(MCIERR_OUTOFRANGE);

    optbDataPosition(lpstrData, bDataFormat, bHours, bMinutes, bSeconds, bFrames);

    return (MCIERR_NO_ERROR);
}


/****************************************************************************
 * Function: DWORD optbDataToMciTimeFormat - Convert a ViSCA data structure
 *               to an MCI time value.
 *
 * Parameters:
 *
 *      int iInst - Instance on whose behalf conversion
 *                       is being done.
 *
 *      LPSTR lpstrData - ViSCA data structure to be converted.
 *
 *      DWORD FAR * lpdwTime - Pointer to DWORD to hold result.
 *
 * Returns: an MCI error code.
 *
 *       Converts a ViSCA 5-byte position data structure to an MCI DWORD
 *       position variable in the current MCI time format (specified in
 *       pinst[iInst].dwTimeFormat).
 ***************************************************************************/
DWORD FAR PASCAL
optbDataToMciTimeFormat(int iInst, BOOL fTimecode, LPSTR lpstrData, DWORD FAR *lpdwTime)
{
    UINT    uHours   = VISCAHOURS(lpstrData);
    UINT    uMinutes = VISCAMINUTES(lpstrData);
    UINT    uSeconds = VISCASECONDS(lpstrData);
    UINT    uFrames  = VISCAFRAMES(lpstrData);
    UINT	uDevFPS = 30;
    UINT    uMCIFPS  ;
    DWORD   dwTimeFormat;
   
    if(fTimecode)
        dwTimeFormat =  pinst[iInst].dwTimeFormat;
    else
        dwTimeFormat =  pinst[iInst].dwCounterFormat;

    uMCIFPS  = optbMciFPS(dwTimeFormat);

    //
    // If a device will return a bogus position. 
    //
    if ((uMinutes >= 60) || (uSeconds >= 60))
    {
        DPF(DBG_ERROR, "Bad uMinutes, uSeconds!\n");
        return (MCIERR_DRIVER_INTERNAL);
    }

    switch(dwTimeFormat)
    {
        case MCI_FORMAT_MILLISECONDS:
            *lpdwTime = (uHours * 3600000L) + (uMinutes * 60000L) +
                        (uSeconds * 1000L) +
                        (uFrames * 1000L / uDevFPS);
            return (MCIERR_NO_ERROR);

        case MCI_FORMAT_HMS:
            *lpdwTime = MCI_MAKE_HMS(uHours, uMinutes, uSeconds);
            return (MCI_COLONIZED3_RETURN);

        case MCI_FORMAT_MSF:
            *lpdwTime = MCI_MAKE_MSF((uHours * 60) + uMinutes, uSeconds,
                                                               uFrames);
            return (MCI_COLONIZED3_RETURN);

        case MCI_FORMAT_TMSF:
            *lpdwTime = MCI_MAKE_TMSF(1, (uHours * 60) + uMinutes, uSeconds,
                                                                   uFrames);
            return (MCI_COLONIZED4_RETURN);

        case MCI_FORMAT_FRAMES:
        case MCI_FORMAT_SAMPLES:
            *lpdwTime = ((uHours * 3600L + uMinutes * 60L + uSeconds) *
                         uDevFPS) + uFrames;
            return (MCIERR_NO_ERROR);

        case MCI_FORMAT_SMPTE_30DROP:
        case MCI_FORMAT_SMPTE_24:
        case MCI_FORMAT_SMPTE_25:
        case MCI_FORMAT_SMPTE_30:
        {
            uFrames  = MulDiv(uFrames, uMCIFPS, uDevFPS);
            //
            // Because of rounding, it's theoretically possible that uFrames
            // will exceed uMCIFPS - 1.  So check for this condition.
            //
            if (uFrames >= uMCIFPS) 
                uFrames = uMCIFPS - 1;
            
            *lpdwTime = ((DWORD)uHours) | ((DWORD)uMinutes << 8) |
                        ((DWORD)uSeconds << 16) | ((DWORD)uFrames << 24);
            return (MCI_COLONIZED4_RETURN);
        }

        default:
            return (MCIERR_BAD_TIME_FORMAT);
    }
}


DWORD FAR PASCAL
optbDataToMciTimeFormatEx(int iInst, BOOL fTimecode, LPSTR lpstrData, DWORD FAR *lpdwTime, DWORD mciTimeFormat)
{
    UINT    uHours   = VISCAHOURS(lpstrData);
    UINT    uMinutes = VISCAMINUTES(lpstrData);
    UINT    uSeconds = VISCASECONDS(lpstrData);
    UINT    uFrames  = VISCAFRAMES(lpstrData);
    UINT	uDevFPS = 30;
    UINT    uMCIFPS  ;
    DWORD   dwTimeFormat;
        
    dwTimeFormat = mciTimeFormat;

    uMCIFPS  = optbMciFPS(dwTimeFormat);

    //
    // If a device will return a bogus position. 
    //
    if ((uMinutes >= 60) || (uSeconds >= 60))
    {
        DPF(DBG_ERROR, "Bad uMinutes, uSeconds!\n");
        return (MCIERR_DRIVER_INTERNAL);
    }

    switch(dwTimeFormat)
    {
        case MCI_FORMAT_MILLISECONDS:
            *lpdwTime = (uHours * 3600000L) + (uMinutes * 60000L) +
                        (uSeconds * 1000L) +
                        (uFrames * 1000L / uDevFPS);
            return (MCIERR_NO_ERROR);

        case MCI_FORMAT_HMS:
            *lpdwTime = MCI_MAKE_HMS(uHours, uMinutes, uSeconds);
            return (MCI_COLONIZED3_RETURN);

        case MCI_FORMAT_MSF:
            *lpdwTime = MCI_MAKE_MSF((uHours * 60) + uMinutes, uSeconds,
                                                               uFrames);
            return (MCI_COLONIZED3_RETURN);

        case MCI_FORMAT_TMSF:
            *lpdwTime = MCI_MAKE_TMSF(1, (uHours * 60) + uMinutes, uSeconds,
                                                                   uFrames);
            return (MCI_COLONIZED4_RETURN);

        case MCI_FORMAT_FRAMES:
        case MCI_FORMAT_SAMPLES:
            *lpdwTime = ((uHours * 3600L + uMinutes * 60L + uSeconds) *
                         uDevFPS) + uFrames;
            return (MCIERR_NO_ERROR);

        case MCI_FORMAT_SMPTE_30DROP:
        case MCI_FORMAT_SMPTE_24:
        case MCI_FORMAT_SMPTE_25:
        case MCI_FORMAT_SMPTE_30:
        {
            uFrames  = MulDiv(uFrames, uMCIFPS, uDevFPS);
            //
            // Because of rounding, it's theoretically possible that uFrames
            // will exceed uMCIFPS - 1.  So check for this condition.
            //
            if (uFrames >= uMCIFPS) 
                uFrames = uMCIFPS - 1;
            
            *lpdwTime = ((DWORD)uHours) | ((DWORD)uMinutes << 8) |
                        ((DWORD)uSeconds << 16) | ((DWORD)uFrames << 24);
            return (MCI_COLONIZED4_RETURN);
        }

        default:
            return (MCIERR_BAD_TIME_FORMAT);
    }
}

DWORD FAR PASCAL
optbDataToMiliseconds(int iInst, BOOL fTimecode, LPSTR lpstrData, DWORD FAR *lpdwTime)
{
    UINT    uHours   = VISCAHOURS(lpstrData);
    UINT    uMinutes = VISCAMINUTES(lpstrData);
    UINT    uSeconds = VISCASECONDS(lpstrData);
    UINT    uFrames  = VISCAFRAMES(lpstrData);
    UINT	uDevFPS = 30;
    DWORD   dwTimeFormat;

    dwTimeFormat =  pinst[iInst].dwTimeFormat;

    //
    // If a device will return a bogus position. 
    //
    if ((uMinutes >= 60) || (uSeconds >= 60))
    {
        DPF(DBG_ERROR, "Bad uMinutes, uSeconds!\n");
        return (MCIERR_DRIVER_INTERNAL);
    }


    *lpdwTime = (uHours * 3600000L) + (uMinutes * 60000L) +
                        (uSeconds * 1000L) +
                        (uFrames * 1000L / uDevFPS);
    return (MCIERR_NO_ERROR);
}


/****************************************************************************
 * Function: DWORD optbMciCloseDriver - Edit instance-specific cleanup.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_GENERIC_PARMS lpGeneric - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_CLOSE_DRIVER
 *       command.
 ***************************************************************************/
static DWORD NEAR PASCAL
optbMciCloseDriver(int iInst, DWORD dwFlags, LPMCI_GENERIC_PARMS lpGeneric)
{     
	int i;     
	 
	if (find_window() == TRUE) {
		for(i=0;i<3;i++)
	            	//if(PostAppMessage(pinst[iInst].hClientApp,VODMCIPaused,0,0L))   
	            	if(PostMessage(hWndClient,VODMCIKill,0,0L))
	            	{
	            		//MessageBox(NULL,"stopaaa!!!",NULL, MB_ICONEXCLAMATION);  
	            		break;
	                }	
	    if (i==3) MessageBox(NULL,"VODMCIKill not sent!!!",NULL, MB_ICONEXCLAMATION);
	}  

    return (optbNotifyReturn(iInst, (HWND) lpGeneric->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
}



/****************************************************************************
 * Function: DWORD optbMciOpenDriver - Edit instance-specific initialization.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_OPEN_PARMS lpOpen - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_OPEN_DRIVER
 *
 ***************************************************************************/
static DWORD NEAR PASCAL
optbMciOpenDriver(int iInst, DWORD dwFlags, LPMCI_DGV_OPEN_PARMS lpOpen)
{                  
	char clientName[200];
    
	if(!(dwFlags & MCI_OPEN_ELEMENT)) {
		_DPF6(DBG_MEM,"ElementName = NULL!!!");
		return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR)); 
    } 
    /*if(_fstrlen((LPSTR)(lpOpen->lpstrElementName))==0) {
		_DPF6(DBG_MEM,"ElementName = [ ]!!!");
		return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR)); 
    }*/
    _DPF6(DBG_MEM,"ElementName is OK!");
      
    _fstrcpy((LPSTR)frs_file_name, (LPSTR)lpOpen->lpstrElementName); 
    if (parse_file(iInst, (LPSTR)(lpOpen->lpstrElementName))==FALSE) {
    	//_DPF7(DBG_MEM,"Error in parse_file");
    	return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_FILE_NOT_FOUND)); 
    }
    
    GetWindowsDirectory((LPSTR)clientName,200);
    _fstrcat(clientName,"\\mvclient.exe ");    //space after name important!

    _fstrcat((LPSTR)clientName,(LPSTR)(lpOpen->lpstrElementName));
    if ((pinst[iInst].hClientApp=WinExec(clientName,SW_SHOWNORMAL))<32) { 
    	if (find_window_kill() == FALSE) {
	    	_DPF7(DBG_MEM,"Error in executing video client (WinExec)"); 
	    	return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_INTERNAL)); 
	    }
	    else {
	    	_DPF7(DBG_MEM,"Just killing video client application. Reopen MCI driver!"); 
	    	return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_DEVICE_NOT_READY)); 
	    }
	    
    }         
           
           //_DPF7(DBG_MEM, "driver_handle in MCI driver: %lu!",(LONG)driver_handle);  
           hWndClient = NULL;

    pinst[iInst].dwTimeFormat     = MCI_FORMAT_FRAMES;
    pinst[iInst].dwCounterFormat  = MCI_FORMAT_FRAMES;
    pinst[iInst].mcimode = MCI_MODE_PLAY; //bartw: MCI_MODE_STOP? 
    optbDataPosition(pinst[iInst].position,VISCADATAHMSF,0,0,0,0);     //bartw!!!
    //optbDataPosition(achData,VISCADATAHMSF,0,1,0,0);
   	//optbDataToMciTimeFormat(iInst, TRUE, achData, 
    //                &(pinst[iInst].length));               //bartw!!! 
    
    
    //bartw: playstartpos and playstarttime used for setting cursor position in Media Player 
    //only in case of long movies having more than 32K frames
    //reason: Obtibase decoder is not handling position properly for bigger values
    slidertime = 0;            
    optbDataToMiliseconds(iInst, TRUE, pinst[iInst].position,
                                                     &(pinst[iInst].playstartpos));
    pinst[iInst].playstarttime = GetTickCount();                   
              
    
    _DPF3(DBG_MEM, "hClientApp=%u!!!\n",(LPSTR)pinst[iInst].hClientApp); 
    
    //There was a trial (below) to put video client code into MCI driver. It required global changes of
    //socket module i.e. socket event notifications couldn't any longer be passed to application message loop!
    
    /*LoadModule 
     //c:\\bartw\\mvclient\\v1.0\\mvclient.exe test.nt
    InitVideoClient();  // Initialise the Optibase Board and Player.

	  pinst[iInst].InitStruct.Ver=MP_CURR_VER;            // ver num for compatibility 
	  pinst[iInst].InitStruct.Flags=0x1;  
	  pinst[iInst].InitStruct.ptr=NULL;
	  
	  //SwitchStackTo(-1,50000,55000);      
	  if (MP_Init(&(pinst[iInst].InitStruct)) ) {
	    MessageBox(NULL, "Cant Initialise Optibase Board!", NULL, MB_ICONEXCLAMATION);
	    loop_terminate = 1;       
	  } 
	  else   
	    MessageBox(NULL, "OK! Initialised Optibase Board!", NULL, MB_ICONEXCLAMATION);   
	  SwitchStackBack();
	  initialise();
           
    lpfnTempGSB = (GETSYSTEM)MakeProcInstance((FARPROC)GetSystemBuffer,hModuleInstance);
    //ReadFromFile("lpszCmdLine!!!"); //Read server IP address and movie name  
    */
    
 
    return (optbNotifyReturn(iInst, (HWND) lpOpen->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
}
  
  
/****************************************************************************
 * Function: DWORD optbMciGetDevCaps - Get device capabilities.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_GETDEVCAPS_PARMS lpCaps - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_GETDEVCAPS
 *       command.
 ***************************************************************************/
static DWORD NEAR PASCAL
optbMciGetDevCaps(int iInst, DWORD dwFlags, LPMCI_GETDEVCAPS_PARMS lpCaps)
{

    if (!(dwFlags & MCI_GETDEVCAPS_ITEM))
        return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_MISSING_PARAMETER));

    switch (lpCaps->dwItem)
    {  

	    case MCI_GETDEVCAPS_CAN_EJECT:
		case MCI_GETDEVCAPS_CAN_RECORD:
		case MCI_GETDEVCAPS_CAN_SAVE:
	    case MCI_DGV_GETDEVCAPS_CAN_FREEZE:
		case MCI_DGV_GETDEVCAPS_CAN_LOCK:
		case MCI_DGV_GETDEVCAPS_CAN_REVERSE:
	    case MCI_DGV_GETDEVCAPS_CAN_STRETCH:   
		case MCI_DGV_GETDEVCAPS_CAN_STR_IN:	   
		case MCI_DGV_GETDEVCAPS_HAS_STILL:
            lpCaps->dwReturn = MAKEMCIRESOURCE(FALSE, MCI_FALSE);
            return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));

	    case MCI_DGV_GETDEVCAPS_CAN_TEST:
        case MCI_GETDEVCAPS_CAN_PLAY:
        case MCI_GETDEVCAPS_COMPOUND_DEVICE:
        case MCI_GETDEVCAPS_HAS_AUDIO:
        case MCI_GETDEVCAPS_HAS_VIDEO:
        case MCI_GETDEVCAPS_USES_FILES:
        case MCI_DGV_GETDEVCAPS_PALETTES:
            lpCaps->dwReturn = MAKEMCIRESOURCE(TRUE, MCI_TRUE);
            return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
         
        case MCI_GETDEVCAPS_DEVICE_TYPE:
            lpCaps->dwReturn = MAKEMCIRESOURCE(MCI_DEVTYPE_DIGITAL_VIDEO, 
                                               MCI_DEVTYPE_DIGITAL_VIDEO);
            return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
		case MCI_DGV_GETDEVCAPS_MAX_WINDOWS:
            lpCaps->dwReturn = MAKEMCIRESOURCE(1,1);  //bartw: I don't know if not zero windows
            return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
        default:
            return (optbNotifyReturn(iInst, (HWND) lpCaps->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_BAD_CONSTANT));
    }
}


/****************************************************************************
 * Function: DWORD optbMciInfo - Get device information.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_INFO_PARMS lpInfo - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_INFO
 *       command.
 ***************************************************************************/
static DWORD NEAR PASCAL
optbMciInfo(int iInst, DWORD dwFlags, LPMCI_INFO_PARMS lpInfo)
{
    if ((dwFlags & MCI_INFO_PRODUCT) && (lpInfo->lpstrReturn != NULL))
    {
        if (lpInfo->dwRetSize == 0L)
            return (optbNotifyReturn(iInst, (HWND) lpInfo->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_PARAM_OVERFLOW));

        if(dwFlags & MCI_TEST)
            return (optbNotifyReturn(iInst, (HWND) lpInfo->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

        lpInfo->lpstrReturn[lpInfo->dwRetSize - 1] = '\0';
       
        LoadString(hModuleInstance, IDS_DEFAULT_INFO_PRODUCT,
                   lpInfo->lpstrReturn, (int)lpInfo->dwRetSize);

        if (lpInfo->lpstrReturn[lpInfo->dwRetSize - 1] == '\0')
            return (optbNotifyReturn(iInst, (HWND) lpInfo->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
        else
            return (optbNotifyReturn(iInst, (HWND) lpInfo->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_PARAM_OVERFLOW));

    }
    else
    {
       return (optbNotifyReturn(iInst, (HWND) lpInfo->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_UNSUPPORTED_FUNCTION));
    }
}

/****************************************************************************
 * Function: DWORD optbNotifyReturn - Notifies an instance (decides if).
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      HWND  hWndNotify  - Window to send notify to.
 *
 *      DWORD dwFlags     - Did the instance actually request notification.
 *
 *      DWORD dwNotifyMsg - Which notification message to send.
 *
 *      DWORD dwReturnMsg - What return value to return.
 *
 * Returns: dwReturnMsg, so you can just return this function.
 *
 *       This function, in a sense, synchronizes the sending of notifies
 *   on a per-instance basis.  I.e. If there is a notify already existing
 *   then it must be superseded because this is a second one. This can
 *   happen because the same or different instance has started a delayed
 *   command and set the notify hwnd when it was started.
 *
 ***************************************************************************/
DWORD FAR PASCAL
optbNotifyReturn(int iInst, HWND hwndNotify, DWORD dwFlags, UINT uNotifyMsg, DWORD dwReturnMsg)
{
    if(dwFlags & MCI_NOTIFY)
    {
        //
        // If return is failure then do not notify at all!
        //
        if(uNotifyMsg == MCI_NOTIFY_FAILURE)
            return dwReturnMsg;

        //
        // If success, or abort, then we must notify now.
        //
        mciDriverNotify(hwndNotify, pinst[iInst].uDeviceID, uNotifyMsg);
    }
    return dwReturnMsg;
}

/****************************************************************************
 * Function: DWORD optbMciStatus - Get device status.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_VCR_STATUS_PARMS lpStatus - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_STATUS
 *       command.
 ***************************************************************************/
DWORD FAR PASCAL
optbMciStatus(int iInst, DWORD dwFlags, LPMCI_DGV_STATUS_PARMS lpStatus)
{
    DPF(DBG_MEM,"Status:Flags=%lx;Callback=%lx;Return=%lx;dwItem=%lx;dwTrack=%lx;lpstrDrive=%lx;dwReference=%lx;",
		dwFlags,
		lpStatus->dwCallback,
		lpStatus->dwReturn,
		lpStatus->dwItem,
		lpStatus->dwTrack,
		lpStatus->lpstrDrive,
		lpStatus->dwReference);
    
  	if (dwFlags & MCI_STATUS_ITEM)
    {
        switch (lpStatus->dwItem)
        {   
        	DWORD dwErr; 
        	
           
            case MCI_STATUS_POSITION:
                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
                              
                if (dwFlags & MCI_STATUS_START)
                {
                    lpStatus->dwReturn = 0;
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
                }    
                                  
                //Special case handling
                //Optibase decoder cannot handle slider position bigger than 32K frames
                //The solution is an estimation of slider position from movie frame rate and real time difference
                {
                      
                    /*if (slidertime + 5000 > 32768 / 30 * 1000)  
	                {
	                	DWORD currtime;
	                	DWORD newposition;    
	                    char estimpos[30];
	                	
	                	currtime = GetTickCount();
						newposition = currtime - pinst[iInst].playstarttime + pinst[iInst].playstartpos;
						optbMilisecondsToViscaData(iInst, TRUE, newposition, estimpos, VISCADATAHMSF);
						dwErr = optbDataToMciTimeFormat(iInst, TRUE, estimpos,
                                                     &(lpStatus->dwReturn));            	
	                } 
	                else*/ //nowe        //normal handling, when Optibase works fine
						dwErr = optbDataToMciTimeFormat(iInst, TRUE, pinst[iInst].position,
                                                     &(lpStatus->dwReturn));              
                }
                                                     
                if(!dwErr || (dwErr == MCI_COLONIZED3_RETURN) || (dwErr == MCI_COLONIZED4_RETURN))
                        return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, dwErr));
                    else
                        return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, dwErr));
                             
   
                
            case MCI_STATUS_LENGTH:
                if ((dwFlags & MCI_TRACK) && (lpStatus->dwTrack != 1))
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_OUTOFRANGE));

                
                //if (pvcr->Port[iPort].Dev[iDev].dwTapeLength != NO_LENGTH)
                //{
                if(pinst[iInst].length != NO_LENGTH)
                {                	
                    if(dwFlags & MCI_TEST)
                        return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
		            //
                    // The user has explicitly set the length of the tape.
                    //                    											
                    //optbDataToMciTimeFormat(iInst, TRUE, pinst[iInst].length,
                    //                                 &(lpStatus->dwReturn)); 
                    lpStatus->dwReturn = pinst[iInst].length;
                 
                    switch (pinst[iInst].dwTimeFormat)
                    {
                        case MCI_FORMAT_HMS:
                        case MCI_FORMAT_MSF:
                            return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_COLONIZED3_RETURN));
                        case MCI_FORMAT_TMSF:
                        case MCI_FORMAT_SMPTE_24:
                        case MCI_FORMAT_SMPTE_25:
                        case MCI_FORMAT_SMPTE_30:
                        case MCI_FORMAT_SMPTE_30DROP:
                            return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback,
                                dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_COLONIZED4_RETURN));

                        default:
                            return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
                    }
                }  
                else      
                {
                	//no handling 
                	break;
                }

            case MCI_STATUS_CURRENT_TRACK:
                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

                lpStatus->dwReturn = 1;
                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

            case MCI_STATUS_NUMBER_OF_TRACKS:
                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

                lpStatus->dwReturn = 1;
                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

            case MCI_STATUS_MODE:
            {
                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
				
				lpStatus->dwReturn = MAKEMCIRESOURCE(pinst[iInst].mcimode,pinst[iInst].mcimode);

                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
            }
           
            case MCI_STATUS_MEDIA_PRESENT:
            {   
            	
                //
                // Determine whether a tape is present by determining the current mode.
                //
                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

                lpStatus->dwReturn = MAKEMCIRESOURCE(TRUE, MCI_TRUE);

                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
            }
            
            case MCI_STATUS_TIME_FORMAT:

                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

                lpStatus->dwReturn = MAKEMCIRESOURCE(pinst[iInst].dwTimeFormat + MCI_FORMAT_MILLISECONDS,
                                            pinst[iInst].dwTimeFormat + MCI_FORMAT_MILLISECONDS_S);
 
                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
            
            case MCI_STATUS_READY:
            {

                if(dwFlags & MCI_TEST)
                    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

 
                lpStatus->dwReturn = MAKEMCIRESOURCE(TRUE, MCI_TRUE);

                return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCI_RESOURCE_RETURNED));
            }   
            
            default:
 				return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_FAILURE, MCIERR_UNSUPPORTED_FUNCTION));
            
        } 
    }
    
    return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags,
                MCI_NOTIFY_FAILURE, MCIERR_MISSING_PARAMETER));
}

/****************************************************************************
 * Function: DWORD optbMciSet - Set various settings.
 *
 * Parameters:
 *
 *      int iInst - Current open instance.
 *
 *      DWORD dwFlags - MCI command flags.
 *
 *      LPMCI_VCR_SET_PARMS lpSet - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called in response to the MCI_SET
 *       command.
 ***************************************************************************/
static DWORD NEAR PASCAL
optbMciSet(int iInst, DWORD dwFlags, LPMCI_DGV_SET_PARMS lpSet)
{    
	int i;
	
	DPF(DBG_MEM,"Set:Flags=%lx;Callback=%lx;TimeFormat=%lx;Audio=%lx;FileFormat=%lx;Speed=%lx;",
		dwFlags,	
		lpSet->dwCallback,
		lpSet->dwTimeFormat,
		lpSet->dwAudio,
		lpSet->dwFileFormat,
		lpSet->dwSpeed);   
    
    if (dwFlags & MCI_SET_AUDIO) {  
    	if (dwFlags & MCI_SET_OFF) { 
    		if (find_window() == TRUE) {
	    		for(i=0;i<3;i++)
	            	if(PostMessage(hWndClient,VODMCISetAudio, 0,0L))
	            		break;
	            if (i==3) MessageBox(NULL,"VODMCISetAudio not sent!!!",NULL, MB_ICONEXCLAMATION); 
            	//else MessageBox(NULL,"VODMCISetAudio sent!!!",NULL, MB_ICONEXCLAMATION); 
            } 
		}
		if (dwFlags & MCI_SET_ON) { 
			if (find_window() == TRUE) {
	    		for(i=0;i<3;i++)
	            	if(PostMessage(hWndClient,VODMCISetAudio, 0,1L))
	            		break;
	            if (i==3) MessageBox(NULL,"VODMCISetAudio not sent!!!",NULL, MB_ICONEXCLAMATION);  
	            //else MessageBox(NULL,"VODMCISetAudio sent!!!",NULL, MB_ICONEXCLAMATION);
	        }  
		}      
		return (optbNotifyReturn(iInst, (HWND) lpSet->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
	}
            
    	
    if (dwFlags & MCI_SET_TIME_FORMAT)
    {
        if( (lpSet->dwTimeFormat >= MCI_FORMAT_MILLISECONDS) &&
            (lpSet->dwTimeFormat <= MCI_FORMAT_TMSF) &&
            (lpSet->dwTimeFormat != MCI_FORMAT_BYTES))
        {
            //
            // First convert DEVICETAPELENGTH to new time format.
            // We do this by first converting DEVICETAPLENGTH from
            // the current MCI time format to a ViSCA HMSF structure,
            // and then converting back to the new MCI time format.
            //
            if(dwFlags & MCI_TEST)
                return (optbNotifyReturn(iInst, (HWND) lpSet->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));

            if (pinst[iInst].length != NO_LENGTH)
            {
                BYTE    achData[5];

                optbMciTimeFormatToViscaData(iInst, TRUE,
                    pinst[iInst].length,
                    achData, VISCADATAHMSF);

                pinst[iInst].dwTimeFormat = lpSet->dwTimeFormat;

                optbDataToMciTimeFormat(iInst, TRUE, achData, 
                    &(pinst[iInst].length));
            }
            else
            {
                pinst[iInst].dwTimeFormat = lpSet->dwTimeFormat;
            }
            return (optbNotifyReturn(iInst, (HWND) lpSet->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
        }
        else
        {
            return (optbNotifyReturn(iInst, (HWND) lpSet->dwCallback, dwFlags,
                MCI_NOTIFY_FAILURE, MCIERR_BAD_TIME_FORMAT));
        }
        
    }

    return (optbNotifyReturn(iInst, (HWND) lpSet->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
}

static DWORD FAR PASCAL
optbMciWindow(int iInst, DWORD dwFlags, LPMCI_DGV_WINDOW_PARMS lpStatus)
{    
	DPF(DBG_MEM,"WINDOW:Flags=%lx;Callback=%lx;Wnd=%x;CmdShow=%x;Text=%lx;",
		dwFlags,	  
		lpStatus->dwCallback,
	    lpStatus->hWnd,
	    lpStatus->nCmdShow,
	    lpStatus->lpstrText);
    
	  if(dwFlags & MCI_TEST)
            return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
          
      return (optbNotifyReturn(iInst, (HWND) lpStatus->dwCallback, dwFlags, MCI_NOTIFY_SUCCESSFUL, MMSYSERR_NOTSUPPORTED));
     

}

MCI_GENERIC_PARMS Generic = { 0 };

/****************************************************************************
 * Function: DWORD optbMciProc - Process MCI commands.
 *
 * Parameters:
 *
 *      WORD wDeviceID - MCI device ID.
 *
 *      WORD wMessage - MCI command.
 *
 *      DWORD dwParam1 - MCI command flags.
 *
 *      DWORD dwParam2 - Pointer to MCI parameter block.
 *
 * Returns: an MCI error code.
 *
 *       This function is called by DriverProc() to process all MCI commands.
 ***************************************************************************/
DWORD FAR PASCAL
optbMciProc(WORD wDeviceID, WORD wMessage, DWORD dwParam1, DWORD dwParam2)
{
    DWORD           dwRes;
    int             iInst   = (int)mciGetDriverData(wDeviceID);
    DWORD	prev_format;

    //
    // Some nice apps send null instead of structure pointers, give our own if this is the case.
    //
    if(!dwParam2)
        dwParam2 = (DWORD)(LPMCI_GENERIC_PARMS) &Generic;

    if (iInst == -1)
        return (MCIERR_INVALID_DEVICE_ID);   

    
    switch (wMessage)
    {
        //
        // Required Commands 
        //
        case MCI_CLOSE_DRIVER:
            dwRes = optbMciCloseDriver(iInst, dwParam1, (LPMCI_GENERIC_PARMS)dwParam2);
            break;

        case MCI_OPEN_DRIVER:
            dwRes = optbMciOpenDriver(iInst, dwParam1, (LPMCI_DGV_OPEN_PARMS)dwParam2);
            break;

        case MCI_GETDEVCAPS:
            dwRes = optbMciGetDevCaps(iInst, dwParam1, (LPMCI_GETDEVCAPS_PARMS)dwParam2);
            break;

        case MCI_INFO:
            dwRes = optbMciInfo(iInst, dwParam1, (LPMCI_INFO_PARMS)dwParam2);
            break;

        case MCI_STATUS:        
        	find_window();
            dwRes = optbMciStatus(iInst, dwParam1, (LPMCI_DGV_STATUS_PARMS)dwParam2);
            break;
        //
        // Basic Commands 
        //
        case MCI_SET:          
            dwRes = optbMciSet(iInst, dwParam1, (LPMCI_DGV_SET_PARMS)dwParam2);
            break;
            
        case MCI_WINDOW:
            dwRes = MCIERR_UNSUPPORTED_FUNCTION;
            break;
        case MCI_CONFIGURE:
        	return (optbNotifyReturn(iInst, (HWND) (((LPMCI_GENERIC_PARMS)dwParam2)->dwCallback), dwParam1, MCI_NOTIFY_SUCCESSFUL, MCIERR_NO_ERROR));
        	break; 


        //
        // Delayed commands; in mcidelay.c 
        //
        //case MCI_SIGNAL:   //bartw:  ???
        case MCI_SEEK:
        case MCI_PAUSE:
        case MCI_PLAY:
        case MCI_RESUME:
        case MCI_STOP:

            dwRes = optbMciDelayed(wDeviceID, wMessage, dwParam1, dwParam2);
            break;   
        //bartw: video client section!
       	case VCActivate:
       		//MessageBox(NULL,"message VCActivate got from client!!!",NULL, MB_ICONEXCLAMATION); 
       		find_window();
       		break;  
       	case VCPosition:
       		//MessageBox(NULL,"message VCPosition got from client!!!",NULL, MB_ICONEXCLAMATION); 
       		      		
       		prev_format = pinst[iInst].dwTimeFormat;
       		pinst[iInst].dwTimeFormat = MCI_FORMAT_FRAMES;
       			
       		optbMciTimeFormatToViscaData(iInst, TRUE, dwParam2, pinst[iInst].position, VISCADATAHMSF);
       			        		
       		pinst[iInst].dwTimeFormat = prev_format;
            break;  
        case VCStop:
        	pinst[iInst].mcimode = MCI_MODE_STOP;
        	break;
        default:
            dwRes = MCIERR_UNRECOGNIZED_COMMAND;
            break;
    }

    //DPF(DBG_MCI, "<----(exit) optbMciProc: iInst = %d wMessage = %u dwRes = %lx\n", iInst, wMessage, dwRes); 
   
	
    return (dwRes);
}                   

static BOOL parse_line(HFILE optbfile, LPSTR pointer, int buffer_size, char ending)
{
	int index;
	
	index=0;
	
	do{
    	if(_lread(optbfile, (LPSTR)(pointer + index),1)==0) return FALSE;
    	//_DPF4(DBG_MEM, "char = %d\n", pointer[index]);
    	index++;
    }while ((pointer[index-1]!=ending)&&(index<buffer_size));
    if (index == buffer_size)
    	return FALSE; 
    if ((index > 1) && (pointer[index-2]==13)) 
    	index--;               //remove 13 ascii character from the end of the line !!!
    pointer[index-1]='\0'; 
 
 	return TRUE;
}  

static DWORD str_to_dw(LPSTR string)
{          
	int index;
	DWORD result;
	
	index = 0;
	result = 0;
	
	while (string[index]!='\0') 
	{
		result = result * 10 + (string[index]-'0');
		index++;  
		//_DPF4(DBG_MEM, "result = %lu\n", result);
	}
	
	return result;
}


static BOOL parse_file(int iInst, LPSTR lpstrElementName)
{
	HFILE optbfile;
	int index;
	char length[50];
	char offset_s[50];
	char scratch[100];
	BOOL res;
	 
	index = 0;
	optbfile = _lopen((LPSTR)lpstrElementName, READ);
	if (optbfile == HFILE_ERROR)
		return FALSE;
	if ((res = parse_line(optbfile, (LPSTR)scratch, sizeof(scratch),'=')) == FALSE) { //get rid of 'NVSID'&'ServerID='
		_lclose(optbfile);
		return FALSE;     
	}
	if ((res = parse_line(optbfile, (LPSTR)pinst[iInst].hostname, sizeof(pinst[iInst].hostname),'\n')) == FALSE) {
		_lclose(optbfile);
		return FALSE;     
	}         
	if ((res = parse_line(optbfile, (LPSTR)scratch, sizeof(scratch),'=')) == FALSE) { //get rid of 'Filename='
		_lclose(optbfile);
		return FALSE;     
	}
	if ((res = parse_line(optbfile, (LPSTR)pinst[iInst].filename, sizeof(pinst[iInst].filename),'\n')) == FALSE) {
		_lclose(optbfile);
		return FALSE;     
	}
	_DPF6(DBG_MEM, "filename = %s\n", (LPSTR)(pinst[iInst].filename));	  
	if ((res = parse_line(optbfile, (LPSTR)scratch, sizeof(scratch),'=')) == FALSE) { //get rid of 'Offset='
		_lclose(optbfile);
		return FALSE;     
	}
	if ((res = parse_line(optbfile, (LPSTR)offset_s, sizeof(offset_s),'\n')) == FALSE) {
		_lclose(optbfile);
		return FALSE;     
	}     
	//hey, we do not need starting offset here but in mvclient.exe
	//offset = str_to_dw((LPSTR)offset_s);     //this offset from metadatafile is in FRAMES 
	//pinst[iInst].offset = from_frame_to_offset(iInst, offset);  //thus we need to do conversion
	//_DPF6(DBG_MEM, "offset = %lu \n", pinst[iInst].offset);
	
	if ((res = parse_line(optbfile, (LPSTR)scratch, sizeof(scratch),'\n')) == FALSE) { //get rid of 'Filesize=something'
		_lclose(optbfile);
		return FALSE;     
	}  
	if ((res = parse_line(optbfile, (LPSTR)scratch, sizeof(scratch),'=')) == FALSE) {   //get rid of 'Length='
		_lclose(optbfile);
		return FALSE;     
	}
	
	if ((res = parse_line(optbfile, (LPSTR)length, sizeof(length),'\n')) == FALSE) { //assumption is that initial format is FRAMES!
		_lclose(optbfile);
		return FALSE;     
	}  
	pinst[iInst].length = str_to_dw((LPSTR)length); 
	_DPF6(DBG_MEM, "length = %lu \n", pinst[iInst].length);
	
    //MessageBox(NULL,pinst[iInst].hostname,NULL,MB_OK);
    //MessageBox(NULL,pinst[iInst].filename,NULL,MB_OK);
    //MessageBox(NULL,offset,NULL,MB_OK);
    //MessageBox(NULL,length,NULL,MB_OK);
    //MessageBox(NULL,url,NULL,MB_OK);
      
    _lclose(optbfile);
    
     return TRUE;
}  

static DWORD getfromnothing( DWORD lParam )
{
	return ( lParam / 15 * 114599 );
}

DWORD FAR PASCAL from_frame_to_offset(int iInst, DWORD lParam)
{   
	HFILE frs_file;
	char frame_s[20];
	char offset_s[50];
	char scratch[100];
	DWORD frame;
	DWORD offset;
	int i;

	frs_file = _lopen((LPSTR)frs_file_name, READ); 
	if (frs_file == HFILE_ERROR) {
	    _DPF7(DBG_MEM, "Error in open %s.FRS file", frs_file_name);
		return 0L;        
	}
	
	for (i=0; i < 12; i++)  {
		if (parse_line(frs_file, (LPSTR)scratch, sizeof(scratch),'\n') == FALSE)   //i cut header --NVSBEG
			return( getfromnothing(lParam) ); 
		//scratch[40]='\0';
	    //MessageBox(NULL, scratch, NULL, MB_OK);
	    }    
	if (parse_line(frs_file, (LPSTR)frame_s, sizeof(frame_s),' ') == FALSE)
		return( getfromnothing(lParam) );
		frame = str_to_dw((LPSTR)frame_s);
		//MessageBox(NULL, frame_s, NULL, MB_OK);
		
	do{
		if (parse_line(frs_file, (LPSTR)offset_s, sizeof(offset_s),'\n') == FALSE)
		      return( getfromnothing(lParam) );
		offset = str_to_dw((LPSTR)offset_s);
		if (parse_line(frs_file, (LPSTR)frame_s, sizeof(frame_s),' ') == FALSE)
			return( getfromnothing(lParam) );
		frame = str_to_dw((LPSTR)frame_s);
	}
	while ((frame <= lParam) && (frame_s[0] != '-'));   //file ends with --NVSEND 
	
	_lclose(frs_file);   
	//MessageBox(NULL, offset_s, NULL, MB_OK);
	
	return (DWORD) offset;
}
	
BOOL FAR PASCAL find_window(void) {
	int i;
	if (hWndClient == NULL) {
    	hWndClient = FindWindow("Video Client","PC Video Client Demo"); 
    	if (hWndClient == NULL) return FALSE;
        for(i=0;i<3;i++)
            	//if(PostAppMessage(pinst[iInst].hClientApp,VODMCIPaused,0,0L))   
            	if(PostMessage(hWndClient,VODHandle,0,(LONG)driver_handle))
            	{
            		//MessageBox(NULL,"stopaaa!!!",NULL, MB_ICONEXCLAMATION);  
            		break;
                }
        if (i==3)  {
        	MessageBox(NULL,"VODHandle not sent!!!",NULL, MB_ICONEXCLAMATION);
        	return FALSE;
        }
    } 
    return TRUE;
}  

BOOL FAR PASCAL find_window_kill(void) {
	int i;
    	hWndClient = FindWindow("Video Client","PC Video Client Demo"); 
    	if (hWndClient == NULL) return FALSE;
        for(i=0;i<3;i++)
            	//if(PostAppMessage(pinst[iInst].hClientApp,VODMCIPaused,0,0L))   
            	if(PostMessage(hWndClient,VODMCIKill,0,(LONG)driver_handle))
            	{
            		//MessageBox(NULL,"stopaaa!!!",NULL, MB_ICONEXCLAMATION);  
            		break;
                }
        if (i==3)  {
        	MessageBox(NULL,"VODHandle not sent!!!",NULL, MB_ICONEXCLAMATION);
        	return FALSE;
        }
    return TRUE;
}