/**************************************************************************
 *
 *  MCIOPTB.C
 *
 *  MCI Optibase MPEG Driver
 *
 *  Description:
 *
 *      Main Module - Standard Driver Interface and Message Procedures
 *                    DriverProc and DrvOpen and Close Routines.
 *
 *      *1)           mcioptb.c - DriverProc and DriverOpen and Close.
 *       2)           mcicmds.c  - MCI commands.
 *       3)           mcidelay.c - MCI delayed commands (asynchronous)
 *
 * ???Warning: The scanning of system.ini for driver entries is not
 *            recommended, but is done here because of the multiple
 *            devices driver.
 *
 ***************************************************************************/
#define  UNICODE
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include "mmddk.h"
#include <string.h>
#include <ctype.h>
#include "digitalv.h"
#include "mcioptb.h"
#include "common.h"

UINT uProcessCount = 0; //Must be initialized to 0.

//
// The following are per-instance pointers.
//
// This must be initialized each time this DLL maps into a process.
UINT            uCommandTable = (UINT)MCI_NO_COMMAND_TABLE;   // handle to VCR command table
HINSTANCE       hModuleInstance;    // module instance  (different in NT - DLL instances)
POpenInstance   pinst;              // Pointer to use. (For both versions) NT it's per-instance.


/****************************************************************************
 * Function: BOOL MemInitializeInstances - Initialize heap of instances.
 *
 * Returns: TRUE
 *
 ***************************************************************************/
BOOL MemInitializeInstances(void)
{
    int i; 
    BYTE achData[5];

    // Erase any old data.
    _fmemset(pinst, (BYTE)0, sizeof(OpenInstance) * MAX_INSTANCES);

    // (Redundant) Set all In use flags to false.
    for(i = 0; i < MAX_INSTANCES; i++)
    {
        pinst[i].fInUse = FALSE;
        pinst[i].mcimode = MCI_MODE_NOT_READY; //bartw: MCI_MODE_NOT_READY? 
        optbDataPosition(pinst[i].position,VISCADATAHMSF,0,0,0,0); 
        pinst[i].dwTimeFormat = MCI_FORMAT_SMPTE_30;
        pinst[i].dwCounterFormat= MCI_FORMAT_SMPTE_30;
        optbDataPosition(achData,VISCADATAHMSF,0,1,0,0);
        optbDataToMciTimeFormat(i, TRUE, achData, 
                    &(pinst[i].length));
    }
    _DPF4(DBG_MEM, "InitializeInstances - completed successfully");
    return TRUE;
}


/****************************************************************************
 * Function: BOOL MemAllocInstance - Allocate one instance from heap of instances.
 *
 * Returns: TRUE
 *
 ***************************************************************************/
int MemAllocInstance(void)  // Return an offset.
{
    int i;

    for(i = 1; i < MAX_INSTANCES; i++)     //Bartw: ???  inst==0 is OK or not?? 
    {
        if(!pinst[i].fInUse)
            break;
    }
    if(i == MAX_INSTANCES)
        return -1;

    _DPF4(DBG_MEM, "MemAllocInstance - instance %x \n", i);

    pinst[i].fInUse = TRUE;

    // Use offsets only, so return

    return i;
}

/****************************************************************************
 * Function: BOOL MemFreeInstance - Free the instance, return it to heap of instances.
 *
 * Parameters:
 *
 *      int  iInstance       - Instance to free.
 *
 * Returns: TRUE
 *
 ***************************************************************************/
BOOL MemFreeInstance(int iInstance)
{
    _fmemset(&pinst[iInstance], (BYTE)0, sizeof(OpenInstance));
    pinst[iInstance].fInUse = FALSE;

    _DPF4(DBG_MEM, "MemFreeInstance - instance %d \n", iInstance);
    return TRUE;
}


/****************************************************************************
 * Function: int LibMain - Library initialization code.
 *
 * Parameters:
 *
 *      HINSTANCE hModInst - Library instance handle.
 *
 *      WORD wDataSeg - Data segment.
 *
 *      WORD cbHeapSize - Heap size.
 *
 *      LPSTR lpszCmdLine - The command line.
 *
 * Returns: 1 if the initialization was successful and 0 otherwise.
 ***************************************************************************/
int FAR PASCAL
LibMain(HINSTANCE hModInst, WORD wDataSeg, WORD cbHeapSize, LPSTR lpszCmdLine)
{
    hModuleInstance = hModInst;
    return (1);
}

/*
 *      WIN16 - makes global pointer point to static data.
 */

//
// In Win3.1 the static variables are allocated here.
//
OpenInstance arRealInst[MAX_INSTANCES];     // The real non-aliased thing.

OpenInstance *MemCreateInstances(void)
{
    pinst =  &arRealInst[0];
    return pinst;
}

/****************************************************************************
 * Function: LRESULT 
  - Respond to the DRV_LOAD message.
 *                 Perform any one-time initialization.
 *
 * Returns: TRUE on success and FALSE on failure.
 ***************************************************************************/
static LRESULT NEAR PASCAL
optbDrvLoad(void)
{
    // In WIN16 this shouldn't make a difference since this only gets called once.
    uProcessCount++; //This is the only shared thing!

   MemCreateInstances();        // This is only once.

   MemInitializeInstances();       // Because the globals will be on a per-instance basis.
    
    // Bartw: to do: memory initialization 
    
    _DPF4(DBG_MEM, "optbDrvLoad - initalized.");

    return ((LRESULT)TRUE);
}


/****************************************************************************
 * Function: LRESULT optbDrvClose - Respond to the DRV_CLOSE message.  Perform
 *     any cleanup necessary each time the device is closed.
 *
 * Parameters:
 *
 *     WORD wDeviceID - The device id being closed.
 *
 * Returns: TRUE on success and FALSE on failure.
 ***************************************************************************/
static LRESULT NEAR PASCAL
optbDrvClose(WORD wDeviceID)
{    
	//Bartw: to do: free memory of this particular instance
    int iInst   = (int)mciGetDriverData(wDeviceID);

     // This cannot be 0
    if(iInst != 0)
       optbInstanceDestroy(iInst);

    _DPF4(DBG_COMM, "DrvClose - completed \n");

    return ((LRESULT)TRUE);
}


/****************************************************************************
 * Function: LRESULT optbDrvFree - Respond to the DRV_FREE message.
 *                 Perform any device shutdown tasks.
 *
 * Returns: TRUE on success and FALSE on failure.
 ***************************************************************************/
static LRESULT NEAR PASCAL
optbDrvFree(WORD wDeviceID)
{
    uProcessCount--; //This is the only shared thing!
    
    if (uProcessCount==0)
    {
	    // If a command table is loaded, then free it.
	    if (uCommandTable != MCI_NO_COMMAND_TABLE)
	    {
	        _DPF4(DBG_MEM, "Freeing table=%u", uCommandTable);
	        mciFreeCommandResource(uCommandTable);
	        uCommandTable = (UINT)MCI_NO_COMMAND_TABLE;
	    } 
	    
    }
    
    _DPF4(DBG_MEM, "DrvFree done");
    return ((LRESULT)TRUE);
}


/****************************************************************************
 * Function: LRESULT optbDrvOpen - Respond to the DRV_OPEN message.  Perform any
 *     initialization which is done once each time the driver is opened.
 *
 * Parameters:
 *
 *      LPWSTR lpstrParams - NULL terminated command line string contains
 *     any characters following the filename in the SYSTEM.INI file.
 *
 *      LPMCI_OPEN_DRIVER_PARMS lpOpen - Pointer to an
 *     MCI_OPEN_DRIVER_PARMS structure with information about this device.
 *
 * Returns: zero on failure or the driver ID that should be passed
 *      to identify this device on subsequent messages.
 *
 *     In this driver, the DRV_OPEN message is used to parse the
 *     parameters string and fill in the device type and custom command
 *     table.  The OPEN_DRIVER message is MCI specific and is used to
 *     register this device with the sample device.
 ***************************************************************************/
static LRESULT NEAR PASCAL
optbDrvOpen(LPSTR lpstrParams, MCI_OPEN_DRIVER_PARMS FAR * lpOpen)
{
    int                 iInst; //Bartw: 0?
     
    // Create each instances thing.
    iInst = optbInstanceCreate(lpOpen->wDeviceID);

    if (iInst == -1) 
        return (0L);

    // Fill in return information
    lpOpen->wCustomCommandTable = uCommandTable;
    lpOpen->wType = MCI_DEVTYPE_DIGITAL_VIDEO;   // ????It will now search other.mci for strings. (as default).
    
    _DPF4(DBG_COMM, "optbDrvOpen - completed \n");

    // this return value will be passed in as dwDriverID in future calls. 
    return (lpOpen->wDeviceID);
}


/****************************************************************************
 * Function: LRESULT optbRemove - Respond to DRV_REMOVE message.
 *
 * Parameters:
 *
 *      HDRVR hDriver - Handle to driver being removed.
 *
 * Returns: TRUE on success, otherwise FALSE.
 ***************************************************************************/
static LRESULT NEAR PASCAL
optbRemove(HDRVR hDriver)
{     

//bartw: to do ???????what about removing entries from system.ini???
    return ((LRESULT)TRUE);
}




extern LRESULT FAR PASCAL DefDriverProc(
    DWORD   dDriverID,
    HANDLE  hDriver,
    UINT    wMessage,
    LONG    lParam1,
    LONG    lParam2);


/***************************************************************************
 * Function: LONG DriverProc - Windows driver entry point.  All Windows driver
 *     control messages and all MCI messages pass through this entry point.
 *                
 * Parameters:
 *
 *      DWORD dwDriverId - For most messages, <p dwDriverId> is the DWORD
 *     value that the driver returns in response to a <m DRV_OPEN> message.
 *     Each time that the driver is opened, through the <f DrvOpen> API,
 *     the driver receives a <m DRV_OPEN> message and can return an
 *     arbitrary, non-zero value. The installable driver interface
 *     saves this value and returns a unique driver handle to the 
 *     application. Whenever the application sends a message to the
 *     driver using the driver handle, the interface routes the message
 *     to this entry point and passes the corresponding <p dwDriverId>.
 *     This mechanism allows the driver to use the same or different
 *     identifiers for multiple opens but ensures that driver handles
 *     are unique at the application interface layer.
 *
 *     The following messages are not related to a particular open
 *     instance of the driver. For these messages, the dwDriverId
 *     will always be zero.
 *
 *         DRV_LOAD, DRV_FREE, DRV_ENABLE, DRV_DISABLE, DRV_OPEN
 *
 *      HDRVR  hDriver - This is the handle returned to the
 *     application by the driver interface.
 *          
 *      UINT uMessage - The requested action to be performed. Message
 *     values below <m DRV_RESERVED> are used for globally defined messages.
 *     Message values from <m DRV_RESERVED> to <m DRV_USER> are used for
 *     defined driver protocols.  Messages above <m DRV_USER> are used
 *     for driver specific messages.
 *
 *      LPARAM lParam1 - Data for this message.  Defined separately for
 *     each message
 *
 *      LPARAM lParam2 - Data for this message.  Defined separately for
 *     each message
 *
 * Returns: Defined separately for each message. 
 ***************************************************************************/
LRESULT CALLBACK _loadds
DriverProc(DWORD dwDriverID, HDRVR hDriver, UINT uMessage, LPARAM lParam1, LPARAM lParam2)
{
    if (uMessage != MCI_STATUS) 
    	_DPF6(DBG_MEM, "Driver got %u message", uMessage); 
    	
    switch (uMessage) {

        case DRV_LOAD:
            // the DRV_LOAD message is received once, when the driver is 
            // first loaded - any one-time initialization code goes here 
            return (optbDrvLoad());

        case DRV_FREE:
            // the DRV_FREE message is received once when the driver is 
            // unloaded - any final shut down code goes here 
            return (optbDrvFree(LOWORD(dwDriverID)));

        case DRV_OPEN:
            // the DRV_OPEN message is received once for each MCI device open 
            if (lParam2) {                  // normal open 
            	driver_handle = hDriver;
                return (optbDrvOpen((LPSTR)lParam1,
                        (LPMCI_OPEN_DRIVER_PARMS)lParam2));
            }
            else {                                  // configuration open
                return (0x00010000);
            }

        case DRV_CLOSE:
            // this message is received once for each MCI device close
            return (optbDrvClose(LOWORD(dwDriverID)));   
            
        case DRV_INSTALL:
            return ((LRESULT)DRVCNF_RESTART);

        case DRV_REMOVE:
            return ((LRESULT)DRVCNF_RESTART);

        case DRV_QUERYCONFIGURE:
            // the DRV_QUERYCONFIGURE message is used to determine if the 
            // DRV_CONCIGURE message is supported - return 1 to indicate 
            // configuration is supported 
            return (0L);
            
        default:
            // all other messages are processed here 

            // select messages in the MCI range 
            if ((!HIWORD(dwDriverID)) && (uMessage >= DRV_MCI_FIRST) &&
                (uMessage <= DRV_MCI_LAST))
            {
                return (optbMciProc(LOWORD(dwDriverID), (WORD)uMessage, lParam1, lParam2));
            }
            else
            {
                // other messages get default processing 
                return (DefDriverProc(dwDriverID, hDriver, uMessage, lParam1, lParam2));
            }
    }                  
    return (0L);
}
