/**********************************************************************
*                          Optibase Inc.                               
*                      All Rights Reserved                             
*              Unpublished and confidential material.                  
*               --- Do not Reproduce or Disclose ---                   
**********************************************************************/
                                                                       
/**********************************************************************
  Module     :  pcmtn.h                                                    
  Writer     :  Amir Aharoni
  Revision   :  2.52
  Description:  PCMTN Decoder DLL definitions module
  Notes      :  
  History    :  Last Modified: 05/04/1994  Alex Zilberstein
**********************************************************************/

typedef unsigned long FRAME;

#define MP_CURR_VER       0x252

#define MP_NTSC_358       0x0001
#define MP_NTSC_443       0x0002
#define MP_PAL            0x0004

/* Maximum size of buffer for mode User buffer...   */
/* User Buffer in System file */
#define READBUFSZ         32768     /* for SYSTEM file */  

/* User Buffer in Video only file */
#define VIDEOBUFSZ        0x5100*2     /* for VIDEO file only */  


typedef BYTE far *(FAR PASCAL  _export *GETVIDEO)(UINT *lpnSize,UINT BuffSize);
typedef BYTE far *(FAR PASCAL  _export *GETSYSTEM)(UINT *lpnSize,UINT BuffSize);

/* Currently not in use. For compatabilty  */
typedef BYTE far *(FAR PASCAL  _export *GETAUDIO)(UINT *lpnSize,UINT BuffSize); 

/*
** MP_STATUS_STRUCT :
** Used by MP_GetStatus() to return the current PCMotion status .
*/

typedef struct
 {
  unsigned int  State;        /* mpeg internal state PLAY,PAUSE etc. */
  unsigned long CurrFrame;    /* current frame number on display     */
  unsigned long LastFrame;    /* last frame number of the video file */
  unsigned int  Status;       /* 1- end of video; 0- other           */
  unsigned long BitRate;
 } MP_STATUS_STRUCT ;


/*
** MP_AUDIO_STATUS_STRUCT :
*/
typedef struct
 {
  int   IsAudio;
  int   Audio_stop;
  int   Volume;
  long  Delay;  /* Currently not in use. For compatabilty  */
 } MP_AUDIO_STATUS_STRUCT ;



/*------------------------------------------------------------------------*/

/*
** MP_INITS_STRUCT :
** used to pass initaliztion values to MP_Init function .
*/
typedef struct
  {
  int     Ver;                /* structure verstion number now: Ver=2.50 */
  char    *ptr;     		      /* Currently not in use. For compatabilty  */
  int     reserve;			   /* with PCMTN dll only.					       */
  unsigned long Flags;        /* 0x1 - NTSC 3.58                         */
                              /* 0x2 - NTSC 4.43                         */
                              /* 0x4 - PAL                               */
                              /* 0x8 - GenLock                           */
                        
  } MP_INIT_STRUCT ;

/*
** MP_INFO_STRUCT :
** Used by MP_GetInfo() to return info of the current MPG file.
*/

typedef struct
 {
  char *FileName;           /* mpeg file name                          */
  DWORD  FileSize;          /* mpeg file size  (KBytes)                */
  int VideoMode;            /* CINEMA(23.976 fps)=1 CINEMA(24 fps)=2   */
                            /* PAL(25 fps)=3                           */  
                            /* NTSC(29.97 fps)=4, NTSC(30 fps)=5       */
  int N;                    /* Currently not in use. For compatabilty  */
  int M;                    /* Currently not in use. For compatabilty  */
  unsigned long BitRate;    /* (KBytes/sec)                            */
  unsigned long FirstFrame; /* first frame number on the movie file    */
  unsigned long LastFrame;  /* last frame number on the movie file     */
  int   IsAudio;            /* 0 - no audio                            */
                            /* 3 - layer 2                             */
  long  Delay;              /* Currently not in use. For compatabilty  */
  UINT Stereo;              /* 0= mono    1= stereo                    */
  UINT Audio_fmt;           /* Audio BitRate (Kbits/sec) for layer 2   */
  UINT Audio_freq;          /* 1 - 11025, 3 - 22050, 5 - 44100 Hz      */
  UINT Audio_length;        /* Currently not in use. For compatabilty  */

 } MP_INFO_STRUCT ;


/*------------------------------------------------------------------------*/

/* return value of mpeg dll functions */
#define MPEG_RC DWORD FAR PASCAL

/* aliases for frame manipulations    */

#define FIRST_FRAME  0L
#define LAST_FRAME   0xFFFFFFFEL
#define CURR_FRAME   0xFFFFFFFFL

#define     FORMAT_VIDEO     1
#define     FORMAT_SYSTEM    2
#define     FORMAT_TGA       4
#define     FORMAT_MPA       5
#define     FORMAT_PMB       6  
#define     FORMAT_YCBCR     7

#define     CONTINUE         0
#define     START            1
#define     STOP            -1
#define     PLAY_MODE        1

#define     STILL_ONLY       0
#define     STILL_AUDIO      1



  /*--------------------------------------------------------------*/
  /* Initialize the PCMTN dll, PCMotion board.                    */
  /* Should be called prior to all PCMTN.DLL calls !              */
  /*--------------------------------------------------------------*/
MPEG_RC MP_Init(MP_INIT_STRUCT *Init);


  /*--------------------------------------------------------------*/
  /* Open MPEG file (system , video or video & audio (layer 2)    */
  /*--------------------------------------------------------------*/
MPEG_RC MP_Open(char *FileName);
  /*--------------------------------------------------------------*/
  /* Fast open MPEG file (system only) for the back to back mode  */
  /*--------------------------------------------------------------*/
MPEG_RC MP_OpenFast(char *FileName);

  /*--------------------------------------------------------------*/
  /* Close current opened file & allow opening of a new file      */
  /*--------------------------------------------------------------*/
MPEG_RC MP_Close(void);

  /*--------------------------------------------------------------*/
  /* Play current file from FromFrame to ToFrame                  */
  /* This function returns immediatly to the user and allows the  */
  /* user to continue working during film playback!               */
  /* Note: FrameRate not currently supported                      */
  /*--------------------------------------------------------------*/
MPEG_RC MP_Play(FRAME FromFrame,FRAME ToFrame,int FrameRate);

  /*--------------------------------------------------------------*/
  /* Pause the film at current position ( during playback )       */
  /*--------------------------------------------------------------*/
MPEG_RC MP_Pause(void);

  /*--------------------------------------------------------------*/
  /* Seeks to the specified Frame number of the film              */
  /*--------------------------------------------------------------*/
MPEG_RC MP_RandomAccess(FRAME Frame);
  /*--------------------------------------------------------------*/
  /* Fast seeks to the First Frame of the film for the back to    */
  /* back mode  (not used for first film )                        */
  /*--------------------------------------------------------------*/
MPEG_RC MP_RandomAccessFast(void);

  /*--------------------------------------------------------------*/
  /* Advance one frame forward from current film position         */
  /*--------------------------------------------------------------*/
MPEG_RC MP_FrameForward(void);

  /*--------------------------------------------------------------*/
  /* Reads current status of PCMotion system into Status          */
  /* (see MP_STATUS_STRUCT)                                       */
  /* This function can be called asyncronically to provide curent */
  /* displayed frame and bit rate.                                */
  /*--------------------------------------------------------------*/
MPEG_RC MP_GetStatus(MP_STATUS_STRUCT *Status);

  /*--------------------------------------------------------------*/
  /*  Set the parameters of the audio file.                       */
  /*--------------------------------------------------------------*/
MPEG_RC MP_SetAudioParam(MP_AUDIO_STATUS_STRUCT *Status);

  /*--------------------------------------------------------------*/
  /*  Get the parameters of the audio file.                       */
  /*--------------------------------------------------------------*/
MPEG_RC MP_GetAudioParam(MP_AUDIO_STATUS_STRUCT *Status);

  /*--------------------------------------------------------------*/
  /*  Get the MPEG file parameters.                               */
  /*--------------------------------------------------------------*/
MPEG_RC MP_GetInfo(MP_INFO_STRUCT *Status);

  /*--------------------------------------------------------------*/
  /* Load image in format TGA or Y,Cb,Cr                          */
  /* x_cut > 0 and y_cut > 0 are start position                   */
  /* for cutting load TGA image without rescalling                */
  /*--------------------------------------------------------------*/
MPEG_RC  MP_DoRedraw(char *szImageFL, int x_cut, int y_cut);

  /*--------------------------------------------------------------*/
  /* Show image after writing the image to the memory             */
  /*--------------------------------------------------------------*/
MPEG_RC  MP_ShowImage(void);

  /*--------------------------------------------------------------*/
  /* Set mode :  STILL_ONLY  (load image only)                    */
  /*             STILL_AUDIO (load image during audio playing)    */
  /*--------------------------------------------------------------*/
MPEG_RC  MP_SetStillMode(int mode);

  /*--------------------------------------------------------------*/
  /* Write data to Dram for User service Y,Cb,Cr                  */
  /* buf - user buffer which hold the Y Cb or Cr buffer           */
  /* row - number of the first row in the buffer                  */
  /* no_of_row - number of rows in the buffer                     */
  /* row_length - row length                                      */
  /* lumaflag - LUMA (0), CB (1) or CR (2)                        */
  /*--------------------------------------------------------------*/
MPEG_RC  MP_WriteDram(unsigned char *buf,unsigned int row,unsigned int no_of_row,
                   unsigned int row_length,int lumaflag);
              
  /*--------------------------------------------------------------*/
  /* Return file format :                                         */
  /*                    FORMAT_VIDEO                              */
  /*                    FORMAT_SYSTEM                             */
  /*                    FORMAT_TGA                                */
  /*                    FORMAT_MPA                                */
  /*                    FORMAT_PMB                                */
  /*                    FORMAT_YCBCR                              */
  /*--------------------------------------------------------------*/
int GetFileType(LPSTR szName);



// ********************************************************************
// Callback Functions:
//
// Purpose: Return pointer to MPEG VIDEO(Video Buffer) or
//                            MPEG SYSTEM(System Buffer).
//
// Syntax:
//    LPSTR FAR PASCAL _export GetVideoBuffer(UINT *lpnSize,UINT BuffSize);
//    LPSTR FAR PASCAL _export GetSystemBuffer(UINT *lpnSize,UINT BuffSize);
//
// Parameters:
//         lpnSize -  Pointer to unsigned integer - number of Bytes in
//                    the returned buffer.
//         BuffSize - Optimum number of bytes requested by the DLL.
// Returns: 
//         Pointer to User buffer.
// Notes:
//        1.User can return a buffer in any size no bigger then the Maximum
//         size of the working mode. (See above READBUFSZ andVIDEOBUFSZ)
//         For best performence we reccomend to return a buffer size equal to 
//         BuffSize.
//        2.GETAUDIO  GetAudioBuffer currently not in use 
// ********************************************************************
MPEG_RC MP_UserParam(GETVIDEO  GetVideoBuffer,
                     GETAUDIO  GetAudioBuffer,
                     GETSYSTEM GetSystemBuffer);

                                  
/********************************************************/
/*    OPTIBASE Error codes  -  Kernel environment       */
/********************************************************/

#define K_MISSING                        1
#define K_SW_ERROR_1                     2
#define K_SW_ERROR_2                     3
#define K_MEM_ERROR                      4
#define K_MEM_WIDTH_ERROR                5
#define K_FILE_OPEN_ERROR                6
#define K_FILE_READ_ERROR                7

/********************************************************/
/*    OPTIBASE Error codes  -  PC environment           */
/********************************************************/

#define NO_ERROR                         0
/* Allocation failure.                               */
#define ALLOC_ERROR                     100
/* Error during disk operations or file not found.  */
#define FILE_ERROR                      101
/* Set command in autoexec.bat not written well. Currently not in use.    */
#define ENV_SET_FAILED                  102

#define PARAM_OUT_OF_RANGE              103
#define MPEG_FILE_ERR                   106
#define MPEG_REF_ERR                    107
#define HARDWARE_FAILURE                108
#define CONFIG_ERROR                    109
#define ILLEGAL_OPERATION               110

/* Error codes as of Version 2.01 */
#define OPEN_MPEG_FILE_ERR              120
#define OPEN_MPEG_REF_ERR               121
#define OPEN_WAVE_FILE_ERR              122
#define WAVE_FILE_ERR                   123
#define BATCH_MODE_ERR                  124

#define USER_DATA_ERR                   133
#define IMAGE_ERROR                     134

